/*
 * pidview.c
 *   for linux-2.6 kernel
 *   written by Y.Ebihara
 *   2012/3/5  Version 1.0
 */

#include <linux/module.h>
#include <linux/sched.h>
#include <linux/kthread.h>
#include <linux/miscdevice.h>
#include <linux/fs.h>
#include <asm/io.h>
#include <asm/uaccess.h>

#define DEVNAME "pidview"

#define PADR 0xA4050120
#define PACR (PADR-0x20)

extern int pid_view1;
extern int pid_view2;
extern int pid_view3;

/* ============================================================ */
/* file operation (miscdev)                                     */
/* ============================================================ */

int pidview_open(struct inode *inode, struct file *filp)
{
	return 0;
}

int pidview_write(struct file *filp, const char *user_buf,size_t count,loff_t *ppos)
{
	char buffer[16];
	int len;
	int ret;

	len = count;
	if(len>sizeof(buffer))
		len=sizeof(buffer);
	ret = copy_from_user(buffer, user_buf, len);
	if(ret)
		return ret;

	if(buffer[0]=='1')
		pid_view1 = current->pid;
	if(buffer[0]=='2')
		pid_view2 = current->pid;
	if(buffer[0]=='3')
		pid_view3 = current->pid;

	return count;
}

static const struct file_operations pidview_fops = {
        .open           = pidview_open,
        .write          = pidview_write,
        .owner          = THIS_MODULE,
};

static struct miscdevice pidview_miscdev = {
        .minor  = MISC_DYNAMIC_MINOR,	// 動的割り当て
        .name   = DEVNAME,
        .fops   = &pidview_fops,
};

/* ============================================================ */
/* 初期化関数                                                   */
/* ============================================================ */

static int pidview_init(void){
	int ret=0;
	printk("%s() hello\n",__func__);

/* ポートA */
	writeb(0xFF, PADR);
	writew(0x5555, PACR);

/* 簡単なドライバは miscdev を使うと楽。udev連動してくれる */
	ret = misc_register(&pidview_miscdev);	
	if(ret){
		printk("misc_register() fail. err %d\n",ret);
		return ret;
	}
	
	printk("  minor=%d\n",pidview_miscdev.minor);

	return 0;
}

/* ============================================================ */
/* 終了関数                                                     */
/* ============================================================ */

static void pidview_exit(void){
	printk("%s() goodby\n",__func__);
	misc_deregister(&pidview_miscdev);
}


// モジュールの初期化関数、終了関数を定義するマクロ
module_init(pidview_init);
module_exit(pidview_exit);

// モジュールライセンス定義マクロ
MODULE_LICENSE("GPL");

