#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdint.h>

#include "lib.h"

void hexdump(void *dump_addr, size_t len, void *print_addr){
	int i;
	unsigned char c;
	char buf[20];
	memset(buf,0,sizeof(buf));

	if(!print_addr)
		print_addr = dump_addr;

	for(i=0; i<len; i++){
		if( (i&0x0f) == 0){
			printf("\n%8p:",print_addr);
		}

		c = *(unsigned char*)dump_addr;
		printf("%02X ",c);
		if(! isprint(c)){
			c='.';
		}
		buf[i & 0x0f]=c;

		dump_addr ++;
		print_addr ++;

		if( ((i&0x0f) == 0x0f) || (i==len-1)){
			printf(": %16.16s",buf);
		}
	}
	printf("\n");
}

/*
 * kernel-3.0.4/Documentation/vm/pagemap.txt
 *
 * Bits 0-54  page frame number (PFN) if present
 * Bits 0-4   swap type if swapped
 * Bits 5-54  swap offset if swapped
 * Bits 55-60 page shift (page size = 1<<page shift)
 * Bit  61    reserved for future use
 * Bit  62    page swapped
 * Bit  63    page present
 */

unsigned long vaddr_to_phys(pid_t pid, void *virt)
{
	int fd;
	off_t offset;
	uint64_t page;
	int page_present;
	int page_swapped;
	int page_shift;
	int pfn;
	unsigned long phys;
	unsigned char buff[8];
	unsigned char filename[64];

	if(pid==0){
		pid = getpid();
	}
	sprintf(filename,"/proc/%d/pagemap",pid);
	fd=open(filename,O_RDONLY);
	if(fd<0){
		perror(filename);
		exit(1);
	}

	offset = (off_t)virt;
	offset >>= PAGE_SHIFT;
	offset *= 8;

	lseek(fd, offset, SEEK_SET);   
	read(fd, buff, 8);
	close(fd);

	page = *(uint64_t*)buff;
	// printf("pid=%d, virt=0x%x, offset=0x%x, page=0x%llx\n",pid, virt, offset, page);

	page_present = (page>>63) & 0x01;
	page_swapped = (page>>62) & 0x01;
	page_shift = (page>>55) & 0x3c;
	pfn = (page) & 0xFFFFFFFF;

	phys = PAGE_OFFSET + (pfn * PAGE_SIZE);
	printf("pid=%d, virt=0x%x, page_present=%d, page_swapped=%d, page_shift=%d, pfn=%d (0x%x), phys=0x%x\n",
		pid, virt, page_present, page_swapped, page_shift, pfn, pfn, phys);

	if(page_present == 0){
		printf("physical page is not present\n");
		phys=0;
	}

	return phys;
}

