#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdint.h>

#include "lib.h"

int main(int argc, char *argv[]){
	int fd;
	unsigned char *mem_map;
	off_t offset;
	size_t length;
	struct stat stat;

	if(! argv[1]){
		printf("usage:\n");
		printf("%s filename\n",argv[0]);
		exit(1);
	}

	/* 第一引数で指定されたファイルをopenする */
	fd=open(argv[1], O_RDWR);
	if(fd<=0){
		perror(argv[1]);
		exit(1);
	}

	/* ファイルサイズの取得 */
	if(fstat(fd, &stat) < 0){
		perror(argv[1]);
		exit(1);
	}
	length = stat.st_size;
	printf("file length = %d\n",length);

	/* fd を使ってmmap する(ポインタが戻る) */
	offset = 0;
	mem_map = mmap(0, length, PROT_READ|PROT_WRITE, MAP_SHARED, fd, offset);
	if(mem_map == MAP_FAILED){
		perror(argv[1]);
		exit(1);
	}
	printf("mmaped at %p\n",mem_map);

    /* 書き換え実験 */

	/* メモリ上でファイルの先頭を "ABC" に書き換え */
	mem_map[0]='A';
	mem_map[1]='B';
	mem_map[2]='C';

	/* メモリ上でファイルの最後を "XYZ" に書き換え */
	mem_map[length-3]='X';
	mem_map[length-2]='Y';
	mem_map[length-1]='Z';

	/* 画面に16進数表示する */
	hexdump(mem_map, length, 0);

	/* 仮想アドレスから物理アドレスを調べる */
	vaddr_to_phys(0, mem_map);

	/* 無限に終わらないように ... */
	while(1){
		printf("pid=%d, sleep...\n",getpid());
		sleep(10);	/* 10秒間スリープ */
	}

	/* mmap を閉じる*/
	munmap((void*)mem_map, length);

	/* デバイスを閉じる*/
	close(fd);
	return 0;
}
