#include <sys/time.h> 
#include <sys/resource.h> 
#include <stdio.h>
#include <malloc.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sched.h>

#include "tpulib.h"

/* ---------------------------------------------------------------------------- */
/* device_mmap                                                                  */
/* ---------------------------------------------------------------------------- */

#define MMAP_BASE 0xa4c90000
#define MMAP_LEN  0x400
int dev_fd;
unsigned char *dev_iomap;

int dev_mmap(void)
{
	unsigned long port;
	printf("this is IOmemory test program 2013-11-22 Y.Ebihara\n");

	/* [foCXopen */
	dev_fd = open(DEVNAME, O_RDWR);
	if(dev_fd < 0){
		perror(DEVNAME);
		exit(1);
	}

	/* fd gmmap (|C^߂)*/
	dev_iomap = mmap(0, MMAP_LEN, PROT_READ|PROT_WRITE, MAP_SHARED, dev_fd, MMAP_BASE);
	if(dev_iomap < 0){
		perror(DEVNAME);
    	exit(1);
	}
	printf("IOmem mmaped at %p\n",dev_iomap);
	return 0;
}

static unsigned char dev_readb(unsigned long addr)
{
	return *(volatile unsigned char*)(dev_iomap + (addr - MMAP_BASE));
}

static unsigned short dev_readw(unsigned long addr)
{
	return *(volatile unsigned short*)(dev_iomap + (addr - MMAP_BASE));
}

static unsigned long dev_readl(unsigned long addr)
{
	return *(volatile unsigned long*)(dev_iomap + (addr - MMAP_BASE));
}

static void dev_writeb(unsigned long addr, unsigned char value)
{
	*(volatile unsigned char*)(dev_iomap + (addr - MMAP_BASE)) = value;
}

static void dev_writew(unsigned long addr, unsigned short value)
{
	*(volatile unsigned short*)(dev_iomap + (addr - MMAP_BASE)) = value;
}

static void dev_writel(unsigned long addr, unsigned long value)
{
	*(volatile unsigned long*)(dev_iomap + (addr - MMAP_BASE)) = value;
}

/* ---------------------------------------------------------------------------- */
/* tpu                                                                          */
/* ---------------------------------------------------------------------------- */



int tpu_init(struct tpudrv_dev *dev){
	dev_writew(dev->base + TPU_TCR,		(2<<5) | dev->pre_scale);		/* TRGBŃNAA 1/1 */
	dev_writew(dev->base + TPU_TMDR,	2);			/* PWM[h */
	// dev_writew(dev->base + TPU_TIOR,	5);					/* High->Low o */
	dev_writew(dev->base + TPU_TIOR,	2);					/* Low->High o */
	dev_writew(dev->base + TPU_TGRB,	dev->interval);		/* C^[oS̎ */
	dev_writew(dev->base + TPU_TGRA,	dev->tgra);			/* RyA}b`   */
	dev_writew(dev->base + TPU_TCNT,	0);				/* JE^   */
	dev_writew(dev->base + TPU_TIER,	0);				/* ݖ */

	return 0;
}

void tpu_start(struct tpudrv_dev *dev)
{
	unsigned int tstr_bit = 1 << (dev->ch);
	unsigned short tstr;

	tstr = dev_readw(TPU_TSTR);
	dev_writew(TPU_TSTR, tstr | tstr_bit);
}

void tpu_stop(struct tpudrv_dev *dev)
{
	unsigned int tstr_bit = 1 << (dev->ch);
	unsigned short tstr;

	tstr = dev_readw(TPU_TSTR);
	dev_writew(TPU_TSTR, tstr & ~tstr_bit);
}

void tpu_start_all(void)
{
	dev_writew(TPU_TSTR,3); // TPU0, TPU1^C}[Jn
}

void tpu_stop_all(void)
{
	dev_writew(TPU_TSTR,0); // TPU0, TPU1^C}[~
}

unsigned short tpu_tcnt(struct tpudrv_dev *dev)
{
	return dev_readw(dev->base +TPU_TCNT);
}



void tpu_set_tgra(struct tpudrv_dev *dev, unsigned short value)
{
	dev_writew(dev->base +TPU_TGRA, value);
	dev->tgra = value;
}


int tpudrv_calc_usec(struct tpudrv_dev *dev, unsigned long cnt)
{
	unsigned long scale,usec;

	scale=1;
	if(dev->pre_scale == 1)
		scale = 4;
	else if(dev->pre_scale == 2)
		scale = 16;
	else if(dev->pre_scale == 3)
		scale = 64;

	usec = cnt;
	usec *= 1000;
	usec /= ((BUS_CLOCK/1000)/scale);

	return usec;
}

int tpu_init_struct(struct tpudrv_dev *dev)
{
	int interval;
	int tpu_hz = dev->hz;
	unsigned int tstr_bit = 1 << (dev->ch);

	interval = BUS_CLOCK/tpu_hz;
	if(interval < 0xffff){
		dev->interval = interval;
		dev->pre_scale = 0;
	}else if(interval < 0x3ffff){
		dev->interval = interval/4;
		dev->pre_scale = 1;
	}else if(interval < 0xfffff){
		dev->interval = interval/16;
		dev->pre_scale = 2;
	}else if(interval < 0x3fffff){
		dev->interval = interval/64;
		dev->pre_scale = 3;
	}else{
		printf("%s() tpu_hz=%d prescale error\n",__func__,tpu_hz);
		return -EINVAL;
	}

	printf("interval =%ld\n",dev->interval);
	printf("pre_scale=%ld\n",dev->pre_scale);

	// dev->tgra = dev->interval/2;
	// dev->tgra = (dev->interval/2) + (dev->interval/4);	// duty 75%
	dev->tgra = dev->interval -1;
	dev->max = 0;
}

int tpu_init_all(struct tpudrv_dev tpudrv_dev[])
{
	tpudrv_dev[0].ch=0;
	tpudrv_dev[0].base=TPU_BASE0;
	tpudrv_dev[0].hz=2000/4/4/4;

	tpudrv_dev[1].ch=1;
	tpudrv_dev[1].base=TPU_BASE1;
	tpudrv_dev[1].hz=2000/4/4/4;

	tpudrv_dev[2].ch=2;
	tpudrv_dev[2].base=TPU_BASE2;

	tpudrv_dev[3].ch=3;
	tpudrv_dev[3].base=TPU_BASE3;

	tpu_init_struct(&tpudrv_dev[0]);
	tpu_init_struct(&tpudrv_dev[1]);

	// tpu_start(&tpudrv_dev[0]);
	// tpu_start(&tpudrv_dev[1]);

	return 0;
}
