#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdint.h>

#include "lib.h"

#define DEVMEM	"/dev/mem"

int main(int argc, char *argv[]){
	int fd;
	unsigned char *mem_map;
	pid_t pid;
	unsigned long vaddr;
	unsigned long phys;
	size_t length=4096;

	if(! argv[2]){
		printf("usage:\n");
		printf("%s pid  virt_addr  [length]\n",argv[0]);
		exit(1);
	}

	/* 第一引数は pid番号 */
	if(argv[1]){
		pid = strtoul(argv[1], NULL, 0);
	}

	/* 第二引数は仮想アドレスの指定(vaddr) */
	if(argv[2]){
		vaddr = strtoul(argv[2], NULL, 0);
	}

	/* 第三引数は表示する長さ。省略時は 4096バイト */
	if(argv[3]){
		length = strtoul(argv[3], NULL, 0);
	}

	if(pid==0){
		exit(1);
	}
	if(vaddr==0){
		exit(1);
	}

	vaddr &= ~(PAGE_SIZE-1);	// PAGEアライメントを取る
	phys = vaddr_to_phys(pid, (void*)vaddr);
	if(phys==0){
		exit(1);
	}

	if(pid==0){
		exit(1);
	}
	if(phys==0){
		exit(1);
	}

	/* ファイルをopenする */
	fd=open(DEVMEM, O_RDONLY);
	if(fd<=0){
		perror(DEVMEM);
		exit(1);
	}

	/* fd を使ってmmap する(ポインタが戻る) */
	mem_map = mmap(0, length, PROT_READ, MAP_SHARED, fd, phys);
	if(mem_map == MAP_FAILED){
		perror(argv[1]);
		exit(1);
	}

	hexdump(mem_map, length, (void*)vaddr);

	/* mmap を閉じる*/
	munmap((void*)mem_map, length);

	/* デバイスを閉じる*/
	close(fd);
	return 0;
}
