/*
 *
 */

#include "config.h"

OUTPUT_FORMAT("elf32-sh-linux", "elf32-sh-linux", "elf32-sh-linux")
OUTPUT_ARCH(sh)

MEMORY
{
	ROM (rx)	: org = ROM_START,  len = ROM_BOOT_SIZE
	RAM (rw)	: org = RAM_START,  len = RAM_SIZE
	RAM_STACK (rw)	: org = STACK_START,len = BOOT_STACK_SIZE
}

SECTIONS
{

/* --------
 * .romtextセクション  起動部分プログラムコード
 *  0x80000000 番地になる
 *   配置アドレス  ROMの先頭
 *   実効アドレス  ROMの先頭
 */

	__boottext = . ; 	
	.boottext :
	{
		cat724/entry.o(.text)
		cat724/cpg_init.o(.text)
		cat724/lowlevel_init.o(.text)
	} > ROM
	__boottext_end = . ; 	

/* --------
 * .textセクション 全体プログラムコード
 *   配置アドレス  ROMの続き
 *   実効アドレス  RAM
 */

    . = ALIGN(4);
	__text_rom_start = . ; 
	.text : AT ( __text_rom_start )
	{
		__text_start = . ; 
		*(.text.startup)
		main.o(.text)
		*(.text)
	} > RAM
	. = __text_rom_start + SIZEOF(.text) ;
	__text_rom_end = . ;


/* --------
 * リードオンリデータセクション
 *     配置アドレス  ROMの続き
 *     実効アドレス  RAM
 */
    . = ALIGN(4);
	__rodata_rom_start = . ; 
    .rodata : AT (__rodata_rom_start)
    {
		__rodata_start = . ; 	
		*(.rodata*) 
		/* こう書くと古いgccでうまく行かない *(.rodata.*)  */
    } > RAM
	. = __rodata_rom_start + SIZEOF(.rodata) ;
	__rodata_rom_end = . ;


/* --------
 * データセクション (初期値付データ)
 *     配置アドレス  ROMの続き
 *     実効アドレス  RAM
 */

    . = ALIGN(4);
    __data_rom_start = . ;
    .data   :  AT(__data_rom_start)
    {
       __data_start = . ;
        *(.data)
    } > RAM
	. = __data_rom_start + SIZEOF(.data) ;
    __data_rom_end = . ;

/* --------
 * .bssセクション
 *     配置アドレス  RAM
 *     実効アドレス  RAM
 */

    . = ALIGN(4);
    .bss : 
    {
	    __bss_start = . ;
        *(.bss)
        *(COMMON)
            . = ALIGN(4);
	    __bss_end = . ;
    } > RAM

/* --------
 * 以下のセクションは無くなる
 */

    .comment        0 : { *(.comment        ) }
    .debug_aranges  0 : { *(.debug_aranges  ) }
    .debug_pubnames 0 : { *(.debug_pubnames ) }
    .debug_abbrev   0 : { *(.debug_abbrev   ) }
    .debug_info     0 : { *(.debug_info     ) }
    .debug_line     0 : { *(.debug_line     ) }
    .debug_frame    0 : { *(.debug_frame    ) }
    .debug_loc      0 : { *(.debug_loc      ) }
    .debug_str      0 : { *(.debug_str      ) }
    .debug_ranges   0 : { *(.debug_ranges   ) }
    .note.gnu.build-id 0: { *(.note.gnu.build-id )}
}
