/*
 * CAT724
 */

/*=================================
  編集可能定数
  =================================*/

/*=================================
  メモリマップ定義
  ROM  エリア0
  RAM  エリア2,3,4,5
  =================================*/

/* 物理的なRAM開始番地(不可変) */
#define PHYS_RAM_START  (0x88000000)       /* 物理メモリ開始番地 */
#define PHYS_RAM_SIZE   (256*1024*1024)    /* 物理メモリ量 256Mbyte */


/* ブートローダのメモリマップ(可変) */
#define BOOT_OFFSET      (128*1024*1024)   /* ブートローダのRAM開始位置 */
#define BOOT_STACK_SIZE  (2*1024*1024)     /* スタックサイズ 2M*/


/* 以下は不可変定義 */
#define RAM_START    (PHYS_RAM_START+BOOT_OFFSET)  /* 0x9000 0000 */
#define RAM_SIZE     (PHYS_RAM_SIZE-BOOT_OFFSET)   /* 128Mbyte    */

#define STACK_START  (PHYS_RAM_START + PHYS_RAM_SIZE - BOOT_STACK_SIZE)

/*
 * RAM MEMORY MAP
 *


8800 0000      ------------------------- +0
PHYS_RAM_START |                       |
               |                       |
               |     Linux Erea        |
               |                       |
               |                       |
               |                       |
               |                       |
               |                       |
9000 0000      ------------------------- +128M  BOOT_OFFSET
RAM_START      |bootloader code shadow |
               |   .txt                |
               -------------------------
               |   .data               |
               -------------------------
               |   .bss                |
               -------------------------
               |                       |
               |      空きRAM          |
               |   (malloc用ヒープ)    |
               |                       |
97E0 0000      -------------------------
STACK_START    |stack  ↑              |
               |(BOOT_STACK_SIZE = 2M) |
               |                       |
9800 0000      ------------------------ +256M  PHYS_RAM_SIZE

 */


/* ROM開始番地 */
#define ROM_START 0x80000000
#define ROM_BOOT_SIZE (128*1024)   /* ROMサイズ 128K */

/* カーネルパラメータアドレス */
#define KERNEL_PARAMETER_ADDRESS   (PHYS_RAM_START + 0x1000 + 0x100)

/*=================================
  編集可能不可能定数
  =================================*/

/* ---------------------------------------------------------
 クロック関連
   ---------------------------------------------------------*/

#define EXTAL_CLK		33333333	/* 発信器 */
#define CPU_CLOCK		(EXTAL_CLK*15)	/* CPU CLOCK 500MHz */
#define MODULE_CLOCK	(EXTAL_CLK*5/4)	/* モジュールクロック */

#define HZ		100		/* 割り込み周期 Hz */
