/* ==============================================================================
 *   SH4 easy ブートローダー Version1.00
 * ============================================================================== */

#include "cpu.h"
#include "config.h"

/* ==============================================================================
 *    SH4スタートアップ  （ここがリセット解除番地 = 通称ゼロ番地 0xA0000000）
 * ============================================================================== */

	.global	_start
	.align 2
_from_start:

/*
 *  キャッシュの初期化(無効)
 */

	mov.l CCR_A,r1
	mov.l CCR_D_DISABLE,r2
	mov.l r2,@r1
	// mov.l CCR_D_ENABLE,r2
	// mov.l r2,@r1

/*
 *  SRを初期化する。
 *  MD=1, RB=0, BL=0, I3～I0=e fはstubのみ
 */

	mov.l INIT_SR,r0
	ldc   r0,sr
	mov   #0x01,r0
	ldc   r0,r7_bank
	/*
	 * スタックをSTACKTOPに初期化する
	 */
	mov.l  STACK_A, r15

/*
 *  cpg_init を呼び出す
 *    クロック逓倍/分周の設定
 */

start_00:
	mov.l   CPG_INIT_A, r0
	jsr     @r0
	 nop
start_01:

/*
 *  bsr_init を呼び出す
 *    RAMを使用可能にするためのバスコントローラの初期化等を行う。
 */

start_10:
	mov.l   BSR_INIT_A, r0
	jsr     @r0
	 nop
start_11:

/*
 *  .textセクションをRAMにコピーする
 *    ROM領域  __text_rom_start から __text_rom_end までを
 *    RAM領域  __text_start にコピーする
 */

start_20:
	mov.l   TEXT_ROM_START_A,r1
	mov.l   TEXT_ROM_END_A,r2
	cmp/eq  r1,r2
	bt      start_22
	mov.l   TEXT_START_A,r0
start_21:	
	mov.l   @r1+,r4
	mov.l   r4,@r0
	cmp/hi  r1,r2
	add     #4,r0
	bt      start_21
start_22:

/*
 *  .rodataセクションをRAMにコピーする
 */

start_30:
	mov.l   RODATA_ROM_START_A,r1
	mov.l   RODATA_ROM_END_A,r2
	cmp/eq  r1,r2
	bt      start_32
	mov.l   RODATA_START_A,r0
start_31:	
	mov.l   @r1+,r4
	mov.l   r4,@r0
	cmp/hi  r1,r2
	add     #4,r0
	bt      start_31
start_32:

/*
 *  .dataセクションをRAMにコピーする
 */

start_40:
	mov.l   DATA_ROM_START_A,r1
	mov.l   DATA_ROM_END_A,r2
	cmp/eq  r1,r2
	bt      start_42
	mov.l   DATA_START_A,r0
start_41:	
	mov.l   @r1+,r4
	mov.l   r4,@r0
	cmp/hi  r1,r2
	add     #4,r0
	bt      start_41
start_42:

/* 
 * bssセクションをゼロクリア
 */

start_50:
	mov.l	BSS_START_A,r0
	mov.l	BSS_END_A,r1
	cmp/eq  r0,r1
	bt      start_52
	mov	#0,r2
start_51:
	mov.l	r2,@r0
	add	#4,r0
 	cmp/hi	r0,r1
	bt	start_51
start_52:

start_60:
	mov.l	BASE_VBR_A,r0
	ldc	r0, vbr

start_70:
	/*	
         *  C言語で書かれたmain()へ
         *    main() は RAMに配置されているので、この先はRAM動作
	 */
	mov.l	C_LANG_MAIN_A,r0
	jsr	@r0
	 nop


	.align 4
CCR_A:	
	.long _CCR
CCR_D_DISABLE:	
	.long CCR_DISABLE
CCR_D_ENABLE:	
	.long CCR_OC_ENABLE
INIT_SR:	
	.long  0x40000000 + 0x0f << 4
STACK_A:	
	.long   RAM_START+RAM_SIZE


TEXT_ROM_START_A:	
	.long 	__text_rom_start
TEXT_ROM_END_A:	
	.long   __text_rom_end
TEXT_START_A:	
	.long  	__text_start

RODATA_ROM_START_A:
	.long 	__rodata_rom_start
RODATA_ROM_END_A:	
	.long   __rodata_rom_end
RODATA_START_A:	
	.long  	__rodata_start

DATA_ROM_START_A:	
	.long 	__data_rom_start
DATA_ROM_END_A:	
	.long   __data_rom_end
DATA_START_A:	
	.long  	__data_start

BSS_START_A:	
	.long   __bss_start
BSS_END_A:	
	.long  	__bss_end

BASE_VBR_A:
	.long	BASE_VBR

CPG_INIT_A:
	.long   cpg_init

BSR_INIT_A:
	.long   bsr_init

C_LANG_MAIN_A:	
	.long	main
