/*=========================================================================
 *  例外/割り込み出入り口処理ルーチン
 *  SH3/SH4はベクタテーブルを持たず、例外,TRAPAが発生するとVBR+0x100番地を、
 *  内部/外部/NMI割り込みが発生すると、VBR+0x600番地を実行する。
 *    本ブートローダでは例外/割り込みは使用しないので無限ループとする
 *    無限ループを作っておくと何か起こったときにJTAG-ICEでブレイクをかけて
 *    何が起こったか理解できる
 *=========================================================================
 */

#include "config.h"
#include "cpu.h"

/*
 *	VBR に設定する値。VBRは _BASE_VBR番地である。
 */
	.section .text
	.align 2
	.global BASE_VBR
BASE_VBR:
	nop
	nop
	nop

/*
 *  Exception 100 vector
 *  アドレスエラー,スロット不当命令例外、Trapa例外等が発生すると実行される。
 */

	.org   0x0100              /*  _BASE_VBR + 0x0100番地に配置   */
	.align 2
	.global general_exception
general_exception:
	sleep
	bra general_exception	/* 無限ループ */
	nop

/*
 *  Exception 600 vector
 *  割り込みが発生すると実行される。
 */

	.org    0x0600        /* _BASE_VBR + 0x0600番地に配置 */
	.align 2
	.global interrupt
interrupt:
	sleep
	bra interrupt	/* 無限ループ */
	nop
