/*
 * もっとも簡単な bootloader
 */

#include "config.h"
#include "cache.h"
#include "mmu.h"
#include "base_board.h"
#include "sci.h"
#include "timer.h"
#include "string.h"
#include "lib/malloc.h"
#include "stdint.h"
#include "image.h"
#include "zlib/zlib.h"

#define VERSION "1.2"
#define KERNEL_PARAMETER "console=ttySC0,115200 root=/dev/mtdblock2 ro rootfstype=jffs2\0"

extern char __bss_start;
extern char __bss_end;
#define MALLOC_START  ((void*)(&__bss_end))
#define MALLOC_END    ((void*)(STACK_START-1))

#define  OUT_BUFFER_SIZE (8*1024*1024)	/* 展開後のメモリサイズ */


void cpu_sleep(void)
{
	asm ("sleep" : : : "memory");
}

void call(unsigned long addr)
{
	typedef void (linux_start_kernel)(void);
	linux_start_kernel *p = (linux_start_kernel *)addr;
	(*p)();
}

void copy_kernel_param(void)
{
	/* カーネルパラメータの転送 */
	memcpy((unsigned char*)KERNEL_PARAMETER_ADDRESS, (unsigned char *)KERNEL_PARAMETER, sizeof(KERNEL_PARAMETER));
}

/* 起動その１: ROMに書き込まれた vmlinux.bin を RAMへコピーしてjumpする(5Mバイト) */
int boot_vmlinuxbin(void)
{
	memcpy((unsigned char *)(PHYS_RAM_START+0x1000),(unsigned char *)(ROM_START+ROM_BOOT_SIZE), 0x500000);	/* 5,242,880 */
	// (転送先,転送元,バイト数)

	copy_kernel_param();
	puts("Jump to 0x88002000\n");
	call(PHYS_RAM_START+0x2000);
	return 0;
}

/* 起動その２: ROMに書き込まれた zImage に飛び込む */
int boot_zImage(void)
{
	copy_kernel_param();
	puts("Jump to 0x80020000\n");
	call(ROM_START+ROM_BOOT_SIZE);
	return 0;
}

/* 起動その３: ROMに書き込まれた uImageを起動する */
int boot_uboot(void *uaddr)
{
	struct image_header *head = uaddr;
	struct image_header x;
	int ret;
	unsigned long outsize;

	x.ih_magic= __be32_to_cpu(head->ih_magic);
	x.ih_hcrc = __be32_to_cpu(head->ih_hcrc);
	x.ih_time = __be32_to_cpu(head->ih_time);
	x.ih_size = __be32_to_cpu(head->ih_size);
	x.ih_load = __be32_to_cpu(head->ih_load);
	x.ih_ep   = __be32_to_cpu(head->ih_ep);
	x.ih_dcrc = __be32_to_cpu(head->ih_dcrc);
	x.ih_os   = head->ih_os;
	x.ih_arch = head->ih_arch;
	x.ih_type = head->ih_type;
	x.ih_comp = head->ih_comp;
	memcpy(x.ih_name, head->ih_name,IH_NMLEN);
	x.ih_name[IH_NMLEN-1] = 0;

#if 0
	printf("ih_magic=0x%x\n",x.ih_magic);  /* IH_MAGIC 0x27051956 */
	printf("ih_hcrc =0x%x\n",x.ih_hcrc );
	printf("ih_time =0x%x\n",x.ih_time );  /* イメージ生成日 unix time */
	printf("ih_size =0x%x(%d)\n",x.ih_size,x.ih_size );  /* ファイルサイズからimage_header 64byteを引いた数値 */
	printf("ih_load =0x%x\n",x.ih_load );  /* ロードアドレス */
	printf("ih_ep   =0x%x\n",x.ih_ep   );  /* エントリーポイント */
	printf("ih_dcrc =0x%x\n",x.ih_dcrc );
	printf("ih_os   =0x%x\n",x.ih_os   );
	printf("ih_arch =0x%x\n",x.ih_arch );
	printf("ih_type =0x%x\n",x.ih_type );
	printf("ih_comp =0x%x\n",x.ih_comp );  /* 圧縮形式 0-3 */
	printf("name=[%s]\n",x.ih_name);
#endif

/* uImageマジックナンバのチェック*/
	if(x.ih_magic != IH_MAGIC){
		/* invalid uBoot magic */
		puts("err: this is no uBoot image\n");
		return -1;
	}

/* CPUアーキテクチャ種別のチェック */
	if(x.ih_arch != IH_CPU_SH){
		puts("err: this is not SH image\n");
		return -1;
	}

	puts("uBoot image name[");
	puts((char*)x.ih_name);
	puts("]\n");

/* 圧縮種別のチェック */
	switch(x.ih_comp){
	case IH_COMP_NONE:
/* 非圧縮であればmemcpy()するだけ */
		if(x.ih_load == 0)
			break;
		if((void*)x.ih_load == uaddr + sizeof(struct image_header))
			break;
		memcpy((void*)x.ih_load, uaddr + sizeof(struct image_header), x.ih_size);
		break;
	case IH_COMP_GZIP:
/* zlib を使い gzip を展開する */
		outsize = OUT_BUFFER_SIZE;
		ret = uncompress2((void*)x.ih_load, &outsize, uaddr + sizeof(struct image_header), x.ih_size);
		if(ret != Z_OK){
			puts("err: uncompress2() error\n");
			return ret;
		}
		break;
	default:
		puts("err: Compression Type not Support\n");
		return -1;
	}
	copy_kernel_param();
	call(x.ih_ep);
	return 0;
}


//========================================================================================
// ブートローダのメイン関数
//========================================================================================
int main()
{
	cache_init();	/* キャッシュ無効 */
	mmu_init();		/* MMU無効 */
	board_init();	/* ピンファンクション初期化 */
	timer_init();	/* タイマー無効 */
	sci_init();		/* シリアル有効 */
	malloc_init(MALLOC_START, MALLOC_END-MALLOC_START);

	puts("\n\n"
		 "CAT760 easyboot ver " VERSION " build " __DATE__ " " __TIME__ " \n");

	cache_enable();	/* キャッシュ有効 */

	timer_start_ch(1);	    // TMU0_1;

	boot_uboot((void*)(ROM_START+ROM_BOOT_SIZE));
	// boot_zImage();
	// boot_vmlinuxbin();

	/* 起動に失敗した場合 無限ループ（停止） */
	puts("cpu idle...\n");
	while(1){
		cpu_sleep();
	}
}
