/*====================================================================
 * boot loader
 *    ޡ0
 *====================================================================
 */

#include "config.h"
#include "cpu.h"
#include "timer.h"

struct tmu_dev tmu_dev[]={
	{
		.base = _TMU_BASE0_0,
		.tstr = (volatile unsigned char*)_TMU_TSTR0,
		.tstr_bit = 0,
	},{
		.base = _TMU_BASE0_1,
		.tstr = (volatile unsigned char*)_TMU_TSTR0,
		.tstr_bit = 1,
	},{
		.base = _TMU_BASE0_2,
		.tstr = (volatile unsigned char*)_TMU_TSTR0,
		.tstr_bit = 2,
	},{
		.base = _TMU_BASE1_0,
		.tstr = (volatile unsigned char*)_TMU_TSTR1,
		.tstr_bit = 0,
	},{
		.base = _TMU_BASE1_1,
		.tstr = (volatile unsigned char*)_TMU_TSTR1,
		.tstr_bit = 1,
	},{
		.base = _TMU_BASE1_2,
		.tstr = (volatile unsigned char*)_TMU_TSTR1,
		.tstr_bit = 2,
	}
};

#define TMU_NUM (sizeof(tmu_dev[])/sizeof(struct tmu_dev))

/* --------------------------------------------------------------------- */
/* ٥                                                              */
/*   (private)                                                           */
/* --------------------------------------------------------------------- */

void tmu_start_dev(struct tmu_dev *dev)
{
	volatile unsigned char *tstr_reg;
	int bit;

	tstr_reg = dev->tstr;
	bit = dev->tstr_bit;

	*TMU_TCR(dev) &= ~TMU_TCR_UNF;                    /* ׵򥯥ꥢ   */
	*tstr_reg |= (1<<bit);
}

void tmu_stop_dev(struct tmu_dev *dev)
{
	volatile unsigned char *tstr_reg;
	int bit;

	tstr_reg = dev->tstr;
	bit = dev->tstr_bit;

	*TMU_TCR(dev) &= ~TMU_TCR_UNF;                    /* ׵򥯥ꥢ   */
	*tstr_reg &= ~(1<<bit);
}

/*
 * ޡ
 *   32bitȥ󥿥ޤץꥹ1/4ǽޤ
 *   0xFFFFFFFF / (MODULE_CLOCK/4) == 412.316äǰǤ
 */

void tmu_init_dev(struct tmu_dev *dev, int msec)
{
	unsigned long c;
	if(msec){
		c = (MODULE_CLOCK/4/1000) * msec;
	}else{
		c=0xffffffff;
	}

	tmu_stop_dev(dev);
	*TMU_TCR(dev)  = (0);    /* P/4ʬ*/
	*TMU_TCOR(dev) = c;      /* timer constant쥸򥻥å */
	*TMU_TCNT(dev) = c;      /* 󥿡򥻥å */
}

/* --------------------------------------------------------------------- */
/* ٥  (public)                                                    */
/* --------------------------------------------------------------------- */

void timer_irq_disable(int ch)
{
	struct tmu_dev *dev = &tmu_dev[ch];
	*TMU_TCOR(dev) &= ~TMU_TCR_UNIE;                    /* ȯԵ */
}

void timer_irq_enable(int ch)
{
	struct tmu_dev *dev = &tmu_dev[ch];
	*TMU_TCOR(dev) |= TMU_TCR_UNIE;                    /* ȯ */
}

void timer_start_ch(int ch)
{
	struct tmu_dev *dev = &tmu_dev[ch];
	tmu_start_dev(dev);
}
void timer_stop_ch(int ch)
{
	struct tmu_dev *dev = &tmu_dev[ch];
	tmu_stop_dev(dev);
}

volatile unsigned long jiffies;

void timer_init(void)
{
/*
 * ѿ
 */
	jiffies=0;

	tmu_init_dev(&tmu_dev[0],10);	// TMU0_0 , 10msec interval
	tmu_init_dev(&tmu_dev[1],0);	// TMU0_1 , free runnning

	timer_start_ch(0);	    // TMU0_0;

}


/* ޡߥ롼 */
void timer_interrupt0()
{
	*TMU_TCR(&tmu_dev[0]) &= ~TMU_TCR_UNF;                    /* ׵򥯥ꥢ   */
	jiffies++;
}

void usleep(int usec)
{
	/* TMU0  MODULE_CLOCK/4 HZ ǥȥ */
	struct tmu_dev *dev = &tmu_dev[0];
	unsigned long start;
	unsigned long wait;

	start = *TMU_TCNT(dev);
	wait = (MODULE_CLOCK/4);
	wait *= usec;
	wait /= 1000000;

	while(start - *TMU_TCNT(dev) < wait){
			;
	}
	return;
}

void msleep(int msec)
{
	/* TMU0  MODULE_CLOCK/4 HZ ǥȥ */
	struct tmu_dev *dev = &tmu_dev[0];
	unsigned long start;
	unsigned long wait;

	start = *TMU_TCNT(dev);
	wait = (MODULE_CLOCK/4);
	wait *= msec;
	wait /= 1000;

	while(start - *TMU_TCNT(dev) < wait){
			;
	}
	return;
}
