#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <fcntl.h>
#include <sys/mman.h>
#include "zlib/zlib.h"

#include "image.h"
#define  UIMAGE_SIZE (8*1024*1024)	/* uImageをロードするメモリサイズ */
#define  OUT_BUFFER_SIZE (8*1024*1024)	/* 展開後のメモリサイズ */

unsigned char uImage_buffer[UIMAGE_SIZE];	/* uImage をロードするメモリ */
unsigned char out_buffer[OUT_BUFFER_SIZE];  /* vmlinux.bin を展開するメモリ */

#define OUTFILENAME  "/tmp/vmlinux.bin"		/* 出力ファイル名 */

int save_vmlinux(unsigned char *out, int size)
{
	int fd,ret;
	fd = open(OUTFILENAME,O_RDWR|O_CREAT,0644);
	if(fd<0){
		perror(OUTFILENAME);
		exit(1);
	}
	ret=write(fd, out, size);
	close(fd);

	if(ret<0){
		perror(OUTFILENAME);
		exit(1);
	}
	if(ret != size){
		printf("err: write() return value error\n");
		exit(1);
	}

	return 0;	/* success */
}

long uboot(unsigned char *out, unsigned char *uimage, int filesize)
{
	struct image_header *head = (struct image_header *)uimage;
	struct image_header x;
	time_t date;
	char datestr[64];
	int ret;
	long outsize;

	x.ih_magic= __be32_to_cpu(head->ih_magic);
	x.ih_hcrc = __be32_to_cpu(head->ih_hcrc);
	x.ih_time = __be32_to_cpu(head->ih_time);
	x.ih_size = __be32_to_cpu(head->ih_size);
	x.ih_load = __be32_to_cpu(head->ih_load);
	x.ih_ep   = __be32_to_cpu(head->ih_ep);
	x.ih_dcrc = __be32_to_cpu(head->ih_dcrc);
	x.ih_os   = head->ih_os;
	x.ih_arch = head->ih_arch;
	x.ih_type = head->ih_type;
	x.ih_comp = head->ih_comp;
	memcpy(&(x.ih_name), head->ih_name,IH_NMLEN);
	x.ih_name[IH_NMLEN-1] = 0;

/* uImageヘッダを表示する */
	printf("ih_magic=0x%x\n",x.ih_magic);  /* IH_MAGIC 0x27051956 */
	printf("ih_hcrc =0x%x\n",x.ih_hcrc );
	date=x.ih_time;
	ctime_r(&date, datestr);
	printf("ih_time =0x%x, %s",x.ih_time, datestr );  /* イメージ生成日 unix time */
	printf("ih_size =0x%x(%d)\n",x.ih_size,x.ih_size );  /* ファイルサイズからimage_header 64byteを引いた数値 */
	printf("ih_load =0x%x\n",x.ih_load );  /* ロードアドレス */
	printf("ih_ep   =0x%x\n",x.ih_ep   );  /* エントリーポイント */
	printf("ih_dcrc =0x%x\n",x.ih_dcrc );
	printf("ih_os   =0x%x\n",x.ih_os   );
	printf("ih_arch =0x%x\n",x.ih_arch );
	printf("ih_type =0x%x\n",x.ih_type );
	printf("ih_comp =0x%x\n",x.ih_comp );  /* 圧縮形式 0-3 */
	printf("name=[%s]\n",x.ih_name);

/* uImageマジックナンバのチェック*/
	if(x.ih_magic != IH_MAGIC){
		/* invalid uBoot magic */
		puts("err: this is no uBoot image\n");
		return -1;
	}

/* CPUアーキテクチャ種別のチェック */
	if(x.ih_arch != IH_CPU_SH){
		puts("err: this is not SH image\n");
		return -1;
	}

/* ファイルサイズのチェック */
	if(x.ih_size + sizeof(struct image_header) != filesize){
		puts("err: ih_size mismatch\n");
		return -1;
	}

/* 圧縮種別のチェック */
	switch(x.ih_comp){
	case IH_COMP_NONE:
/* 非圧縮であればmemcpy()するだけ */
		outsize=x.ih_size;
		memcpy(out_buffer, uimage + sizeof(struct image_header), outsize);
		ret=0;
		break;
	case IH_COMP_GZIP:
/* zlib を使い gzip を展開する */
		outsize = OUT_BUFFER_SIZE;
		ret = uncompress2(out_buffer, &outsize, uimage + sizeof(struct image_header), x.ih_size);
		if(ret != Z_OK){
			puts("err: uncompress2() error\n");
			return ret;
		}
		break;
	default:
		puts("err: Compression Type not Support\n");
		return -1;
	}


/* 戻り値の表示 */
	printf("ret=%d, outsize=%d",ret,outsize);
	return outsize;	/* success: inflate size */
}

int load_uImage(unsigned char *uimage, char *filename)
{
	/* ファイルをopenする */
	int fd,ret;
	fd=open(filename, O_RDONLY);
	if(fd<=0){
		perror(filename);
		exit(1);
	}
	ret=read(fd, uimage, UIMAGE_SIZE);
	close(fd);
	if(ret<0){
		perror(filename);
		return ret;	/* error */
	}
	if(ret==0){
		printf("err: file is zero byte ?\n");
		return -1;
	}
	if(ret==UIMAGE_SIZE){
		printf("err: UIMAGE_SIZE is not enough\n");
		return -1;
	}
	return ret;	/* 	success: read size */
}

print_help(void){
	printf("a.out uImage_filename\n");
}

int main(int argc, char *argv[]){
	int fd;
	int size;
	
	if(! argv[1]){
		print_help();
		exit(1);
	}

	size = load_uImage(uImage_buffer, argv[1]);
	if(size<0)
		exit(1);

	size = uboot(out_buffer, uImage_buffer, size);
	if(size<0)
		exit(1);

	size = save_vmlinux(out_buffer, size);
	if(size<0)
		exit(1);

	return 0;
}
