#include <sys/time.h> 
#include <sys/resource.h> 
#include <stdio.h>
#include <malloc.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sched.h>

#define DEVNAME "/dev/mem"

#define BUS_CLOCK	83333333	/* 83MHz */
#define TPU_HZ		25			/* 25Hz */

#define TPU_TSTR	0xa4c90000
#define TPU_BASE0	0xa4c90010
#define TPU_BASE1	0xa4c90050
#define TPU_BASE2	0xa4c90090
#define TPU_BASE3	0xa4c900d0
#define TPU_TCR		0x00
#define TPU_TMDR	0x04
#define TPU_TIOR	0x08
#define TPU_TIER	0x0C
#define TPU_TSR		0x10
#define TPU_TCNT	0x14
#define TPU_TGRA	0x18
#define TPU_TGRB	0x1c
#define TPU_TGRC	0x20
#define TPU_TGRD	0x24

/* ---------------------------------------------------------------------------- */
/* device_mmap                                                                  */
/* ---------------------------------------------------------------------------- */

#define MMAP_BASE 0xa4c90000
#define MMAP_LEN  0x400
int dev_fd;
unsigned char *dev_iomap;

static int dev_mmap(void)
{
	unsigned long port;
	printf("this is IOmemory test program 2013-11-22 Y.Ebihara\n");

	/* 物理メモリーデバイスをopenする */
	dev_fd = open(DEVNAME, O_RDWR);
	if(dev_fd < 0){
		perror(DEVNAME);
		exit(1);
	}

	/* fd を使ってmmap する(ポインタが戻る)*/
	dev_iomap = mmap(0, MMAP_LEN, PROT_READ|PROT_WRITE, MAP_SHARED, dev_fd, MMAP_BASE);
	if(dev_iomap < 0){
		perror(DEVNAME);
    	exit(1);
	}
	printf("IOmem mmaped at %p\n",dev_iomap);
	return 0;
}

static unsigned char dev_readb(unsigned long addr)
{
	return *(volatile unsigned char*)(dev_iomap + (addr - MMAP_BASE));
}

static unsigned short dev_readw(unsigned long addr)
{
	return *(volatile unsigned short*)(dev_iomap + (addr - MMAP_BASE));
}

static unsigned long dev_readl(unsigned long addr)
{
	return *(volatile unsigned long*)(dev_iomap + (addr - MMAP_BASE));
}

static void dev_writeb(unsigned long addr, unsigned char value)
{
	*(volatile unsigned char*)(dev_iomap + (addr - MMAP_BASE)) = value;
}

static void dev_writew(unsigned long addr, unsigned short value)
{
	*(volatile unsigned short*)(dev_iomap + (addr - MMAP_BASE)) = value;
}

static void dev_writel(unsigned long addr, unsigned long value)
{
	*(volatile unsigned long*)(dev_iomap + (addr - MMAP_BASE)) = value;
}

/* ---------------------------------------------------------------------------- */
/* tpu                                                                          */
/* ---------------------------------------------------------------------------- */

struct tpudrv_dev{
	unsigned long ch;
	unsigned long base;
	unsigned long hz;
	unsigned long interval;
	unsigned long pre_scale;
	unsigned long tgra;
	unsigned long max;
}tpudrv_dev[4];

static int tpu_init_channel(struct tpudrv_dev *dev)
{
	int interval;
	int tpu_hz = dev->hz;
	unsigned int tstr_bit = 1 << (dev->ch);

	interval = BUS_CLOCK/tpu_hz;
	if(interval < 0xffff){
		dev->interval = interval;
		dev->pre_scale = 0;
	}else if(interval < 0x3ffff){
		dev->interval = interval/4;
		dev->pre_scale = 1;
	}else if(interval < 0xfffff){
		dev->interval = interval/16;
		dev->pre_scale = 2;
	}else if(interval < 0x3fffff){
		dev->interval = interval/64;
		dev->pre_scale = 3;
	}else{
		printf("%s() tpu_hz=%d prescale error\n",__func__,tpu_hz);
		return -EINVAL;
	}

	printf("interval =%ld\n",dev->interval);
	printf("pre_scale=%ld\n",dev->pre_scale);

	dev->tgra = dev->interval/2;
	dev->max = 0;

	dev_writew(dev->base + TPU_TCR,		(2<<5) | dev->pre_scale);		/* TRGBでクリア、 1/1分周 */
	// dev_writew(dev->base + TPU_TIOR,	5);					/* High->Low 出力 */
	dev_writew(dev->base + TPU_TIOR,	2);					/* Low->High 出力 */
	dev_writew(dev->base + TPU_TGRB,	dev->interval);		/* インターバル全体周期 */
	dev_writew(dev->base + TPU_TGRA,	dev->tgra);			/* コンペアマッチ   */
	dev_writew(dev->base + TPU_TMDR,	2);			/* PWMモード */
	dev_writew(dev->base + TPU_TCNT,	0);				/* カウンタ初期化   */
	dev_writew(dev->base + TPU_TIER,	0);				/* 割込み無効 */

	return 0;
}

static void tpu_start(struct tpudrv_dev *dev)
{
	unsigned int tstr_bit = 1 << (dev->ch);
	unsigned short tstr;

	tstr = dev_readw(TPU_TSTR);
	dev_writew(TPU_TSTR, tstr | tstr_bit);
}

static void tpu_stop(struct tpudrv_dev *dev)
{
	unsigned int tstr_bit = 1 << (dev->ch);
	unsigned short tstr;

	tstr = dev_readw(TPU_TSTR);
	dev_writew(TPU_TSTR, tstr & ~tstr_bit);
}

#define TCNT(dev)	*(volatile unsigned short*)(dev_iomap + ((dev->base - MMAP_BASE)+TPU_TCNT))

static int tpudrv_calc_usec(struct tpudrv_dev *dev, unsigned long cnt)
{
	unsigned long scale,usec;

	scale=1;
	if(dev->pre_scale == 1)
		scale = 4;
	else if(dev->pre_scale == 2)
		scale = 16;
	else if(dev->pre_scale == 3)
		scale = 64;

	usec = cnt;
	usec *= 1000;
	usec /= ((BUS_CLOCK/1000)/scale);

	return usec;
}

static int tpu_init(void)
{
	tpudrv_dev[0].ch=0;
	tpudrv_dev[0].base=TPU_BASE0;
	tpudrv_dev[0].hz=2000;

	tpudrv_dev[1].ch=1;
	tpudrv_dev[1].base=TPU_BASE1;
	tpudrv_dev[1].hz=2000;

	tpudrv_dev[2].ch=2;
	tpudrv_dev[2].base=TPU_BASE2;

	tpudrv_dev[3].ch=3;
	tpudrv_dev[3].base=TPU_BASE3;

	dev_writew(TPU_TSTR,0); // TPU0, TPU1タイマー停止

	tpu_init_channel(&tpudrv_dev[0]);
	tpu_init_channel(&tpudrv_dev[1]);

	// tpu_start(&tpudrv_dev[0]);
	// tpu_start(&tpudrv_dev[1]);

	return 0;
}


/* ---------------------------------------------------------------------------- */
/* malloc_test                                                                  */
/* ---------------------------------------------------------------------------- */


int main()
{
	int i;
	char *p;

	unsigned long cnt;
	struct tpudrv_dev *dev;

	puts("hello. malloc_speed1\n");
	dev_mmap();
	tpu_init();

/* リアルタイムプロセス化 */
	struct sched_param sp;
	sp.sched_priority=1;
	sched_setscheduler(0, SCHED_RR, &sp);

// 初回malloc()

	dev = &tpudrv_dev[0];
/* 一旦プロセスを開放しておく */
	usleep(100*1000); // 100msec

// brk()システムコールを含んだ時間計測 */

	tpu_stop(dev);
	TCNT(dev)=0;	// カウンタリセット
	tpu_start(dev);
	malloc(256);	

	cnt = TCNT(dev);
	dev_writew(dev->base + TPU_TGRA, cnt);   /* パルス立ち上げ  */
	// tpu_stop(dev);
	printf("1st cnt=%d: %d usec\n",cnt, tpudrv_calc_usec(dev,cnt));

// ２回目malloc()

	dev = &tpudrv_dev[1];
/* 一旦プロセスを開放しておく */
	usleep(100*1000); // 100msec

// brk()システムコールを含まない時間計測 */

	tpu_stop(dev);
	TCNT(dev)=0;	// カウンタリセット
	tpu_start(dev);
	malloc(256);	

	cnt = TCNT(dev);
	dev_writew(dev->base + TPU_TGRA, cnt);   /* パルス立ち上げ  */
	// tpu_stop(dev);
	printf("2nd cnt=%d: %d usec\n",cnt, tpudrv_calc_usec(dev,cnt));
}
